/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include <Xm/SelectioB.h>

#include "Tab.h"
#include "xmap.h"
#include "gps.h"
#include "values.h"
#include "mode.h"
#include "debug.h"

extern Widget draw_w;
extern void show_help_cb();
extern XmFontList big_fl;
extern void *d_route_p();
extern struct c_values cv;

static Widget navigatedialog;
static Widget bul1, bul2;
static Widget dialog, tab;
static Widget start, cancel, help;
static Widget cog, cts, wpt, dst;
static Widget distunit;
static int new;
static int towp;
static int Fwarning;

void update_route_info()
{
    char bufq[64];
    XmString b;
    double d, dist;
    WP *wp;

    if (gps_data_valid() &&
	(mode_is(XROUTE) && ((cv.update & (U_LATITUDE | U_LONGITUDE | U_UNITS)) || new))) {
	wp = d_route_p(towp);
	if (new) {
	    if (wp->name && *(wp->name)) 
		snprintf(bufq, 63, "%s", wp->name);
	    else
		snprintf(bufq, 63, " %d", towp + 1);

	    b = XmStringCreateSimple(bufq);
	    XtVaSetValues(wpt, XmNlabelString, b, NULL);
	    XmStringFree(b);
	    new = 0;
	}
	snprintf(bufq, 63, " %03.0lf", cv.heading_val);
	b = XmStringCreateSimple(bufq);
	XtVaSetValues(cog, XmNlabelString, b, NULL);
	XmStringFree(b);

	dist = distance(gpsdata.latitude, gpsdata.longitude, wp->lat, wp->lon);
	d = cv_dist(dist);
	if (((int) d) < 10)
	    snprintf(bufq, 63, "%1.2lf", d);
	else if (((int) d) < 100)
	    snprintf(bufq, 63, "%2.1lf", d);
	else if (((int) d) < 10000)
	    snprintf(bufq, 63, "%4.0lf", d);
	else
	    snprintf(bufq, 63, "%3.0lfK", d / 1000);
	b = XmStringCreateSimple(bufq);
	XtVaSetValues(dst, XmNlabelString, b, NULL);
	XmStringFree(b);

	d = heading(gpsdata.latitude, gpsdata.longitude, wp->lat, wp->lon);
	snprintf(bufq, 63, " %03.0lf", d);
	b = XmStringCreateSimple(bufq);
	XtVaSetValues(cts, XmNlabelString, b, NULL);
	XmStringFree(b);

	if (!Fwarning && dist < 0.150) {
	    char bufp[128];

	    Fwarning++;
	    snprintf(bufp, 127, "Aproaching waypoint #%d\n", towp + 1);
	    prox_alarm(bufp);
	}
	if (Fwarning)
	    beep();

    }
}

void clear_route_info()
{
    char bufp[64];
    XmString b;
    double d, dist;

    snprintf(bufp, 63, " %d", 1);
    b = XmStringCreateSimple(bufp);
    XtVaSetValues(wpt, XmNlabelString, b, NULL);
    XmStringFree(b);

    b = XmStringCreateSimple("----");
    XtVaSetValues(cog, XmNlabelString, b, NULL);
    XtVaSetValues(dst, XmNlabelString, b, NULL);
    XtVaSetValues(cts, XmNlabelString, b, NULL);
    XmStringFree(b);

}

void navigate_next(void)
{
    towp++;
    Fwarning = 0;

    if (d_route_p(towp) == NULL) {
	ch_mode(XROUTE, STOP);
	XtPopdown(navigatedialog);
	warning("You have arrived!\n");
    } else
	new = 1;
}

static void ok_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    ch_mode(XROUTE, STOP);
    XtPopdown(navigatedialog);
}

stop_navigate()
{
    if (mode_is(XROUTE)) {
	ch_mode(XROUTE, STOP);
	XtPopdown(navigatedialog);
    }
}

static void cancel_cb(Widget w, XtPointer client_data, XmAnyCallbackStruct * cbs)
{
    ch_mode(XROUTE, STOP);
    XtPopdown(navigatedialog);
}

static void do_navigate()
{
    XmString b;
    Widget fr1, fr2, fr3, w1, w2, w3;

    fr1 = XtVaCreateManagedWidget("fr1", xmFrameWidgetClass, bul1,
				  XmNx, 0,
				  XmNy, 10,
				  XmNwidth, 160,
				  XmNheight, 174,
				  NULL);
    w1 = XtVaCreateManagedWidget("w1", xmBulletinBoardWidgetClass, fr1,
				 XmNmarginHeight, 0,
				 NULL);


    fr2 = XtVaCreateManagedWidget("fr2", xmFrameWidgetClass, bul1,
				  XmNx, 180,
				  XmNy, 10,
				  XmNwidth, 110,
				  XmNheight, 70,
				  NULL);
    w2 = XtVaCreateManagedWidget("w2", xmBulletinBoardWidgetClass, fr2,
				 NULL);

    fr3 = XtVaCreateManagedWidget("fr3", xmFrameWidgetClass, bul1,
				  XmNx, 180,
				  XmNy, 90,
				  XmNwidth, 110,
				  XmNheight, 92,
				  NULL);
    w3 = XtVaCreateManagedWidget("w3", xmBulletinBoardWidgetClass, fr3,
				 NULL);

    XtVaCreateManagedWidget("WPT", xmLabelWidgetClass, w1,
			    XmNx, 10,
			    XmNy, 20,
			    NULL);

    XtVaCreateManagedWidget("CTS", xmLabelWidgetClass, w1,
			    XmNx, 10,
			    XmNy, 60,
			    NULL);

    XtVaCreateManagedWidget("COG", xmLabelWidgetClass, w1,
			    XmNx, 10,
			    XmNy, 100,
			    NULL);

    XtVaCreateManagedWidget("DST", xmLabelWidgetClass, w1,
			    XmNx, 10,
			    XmNy, 140,
			    NULL);

    b = XmStringCreateSimple(cv.distunit);
    distunit = XtVaCreateManagedWidget("nm", xmLabelWidgetClass, w1,
				       XmNx, 130,
				       XmNy, 140,
				       XmNlabelString, b,
				       NULL);
    XmStringFree(b);

    b = XmStringCreateSimple("----");

    wpt = XtVaCreateManagedWidget("wpt", xmLabelWidgetClass, w1,
				  XmNx, 40,
				  XmNy, 5,
				  XmNwidth, 90,
				  XmNfontList, big_fl,
				  XmNlabelString, b,
				  NULL);

    cts = XtVaCreateManagedWidget("cts", xmLabelWidgetClass, w1,
				  XmNx, 40,
				  XmNy, 45,
				  XmNwidth, 90,
				  XmNfontList, big_fl,
				  XmNlabelString, b,
				  NULL);

    cog = XtVaCreateManagedWidget("cog", xmLabelWidgetClass, w1,
				  XmNx, 40,
				  XmNy, 85,
				  XmNwidth, 90,
				  XmNfontList, big_fl,
				  XmNlabelString, b,
				  NULL);

    dst = XtVaCreateManagedWidget("dst", xmLabelWidgetClass, w1,
				  XmNx, 40,
				  XmNy, 125,
				  XmNwidth, 90,
				  XmNfontList, big_fl,
				  XmNlabelString, b,
				  NULL);

    XmStringFree(b);
}

void navigate(Widget w, XtPointer client_data, XtPointer call_data)
{
    Pixel foreground;

    if (!navigatedialog) {

	foreground = get_pixel(w, "Black");
	navigatedialog = XtVaCreatePopupShell("navigatedialog",
				xmDialogShellWidgetClass, GetTopShell(w),
					      XmNtitle, "Navigate",
					      XmNdeleteResponse, XmUNMAP,
					  XmNresizePolicy, XmRESIZE_NONE,
					      XmNnoResize, True,
					      XmNwidth, 340,
					      XmNminWidth, 340,
					      XmNmaxWidth, 340,
					      XmNheight, 270,
					      XmNminHeight, 270,
					      XmNmaxHeight, 270,
					      XmNmarginWidth, 2,
					      XmNmarginHeight, 2,
					      NULL);

	dialog = XtVaCreateWidget("dialog",
			      xmBulletinBoardWidgetClass, navigatedialog,
				  XmNforeground, foreground,
				  XmNautoUnmanage, False,
				  XmNdialogStyle, XmDIALOG_WORK_AREA,
				  NULL);
	XtAddCallback(dialog, XmNunmapCallback, (void *) cancel_cb, NULL);

	tab = XtVaCreateWidget("Tab", xmTabWidgetClass, dialog,
			       XmNx, 0,
			       XmNy, 0,
			       XmNwidth, 320,
			       XmNheight, 220,
			       NULL);

#ifdef GOOD_GEOMETRY_MANAGER

	// Lesstif causes this not to work.
	bul1 = XtVaCreateWidget("Route Status",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	do_navigate();
	XtManageChild(bul1);
#else

    // have to force size
    bul1 = XtVaCreateWidget("Route Status", xmDrawingAreaWidgetClass, tab,
                        //      XmNwidth, 500,
                        //      XmNheight, 500,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
                              NULL);
	do_navigate();
	XtManageChild(bul1);

#endif

#if 0
	bul2 = XtVaCreateWidget("2.",
				xmBulletinBoardWidgetClass, tab,
				XmNforeground, foreground,
				XmNautoUnmanage, False,
				NULL);
	XtManageChild(bul2);
#endif

	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, dialog,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 1,
					 XmNhighlightColor, foreground,
					 XmNwidth, 60,
					 XmNx, 140,
					 XmNy, 240,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, (XtCallbackProc) cancel_cb, NULL);

	start = XtVaCreateManagedWidget("OK",
					xmPushButtonWidgetClass, dialog,
					XmNforeground, foreground,
					XmNhighlightThickness, 1,
					XmNhighlightColor, foreground,
					XmNwidth, 60,
					XmNx, 40,
					XmNy, 240,
					NULL);
	XtAddCallback(start, XmNactivateCallback, (XtCallbackProc) ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, dialog,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 1,
				       XmNhighlightColor, foreground,
				       XmNwidth, 60,
				       XmNx, 240,
				       XmNy, 240,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, (XtCallbackProc) show_help_cb, "navigate.html");

	XtManageChild(tab);
    }
    ch_mode(XROUTE, START);
    new = 1;
    towp = 0;
    Fwarning = 0;
    clear_route_info();
    XtManageChild(dialog);
    XmTabSetTabWidget(tab, bul1, True);
    XtManageChild(navigatedialog);
    XtPopup(navigatedialog, XtGrabNone);
}
